/*
 Copyright (c) 2002  Conectiva Inc.

 Written by Gustavo Niemeyer <niemeyer@conectiva.com>.

 This file is part of rpmver
 
 rpmver is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 pybot is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with pybot; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <rpmlib.h>

#ifndef OLDRPM
#include <rpmds.h>
#endif

#define VERSION "2.1"

static const char usage[] = "\
RPM version comparing tool " VERSION "\n\
\n\
Usage: rpmver [OPTIONS] VER1 [OP] VER2\n\
\n\
Options:\n\
   -v, --verbose  be verbose\n\
   -h, --help     show this message\n\
\n\
Examples:\n\
\n\
	rpmver 1.0 = 1.2-1cl\n\
	rpmver 2:1.0b-3cl '<=' 1.2-1cl\n\
	rpmver -v 0.2-1cl != 0.3-1cl\n\
	rpmver \"0.2-1cl <> 0.3-1cl\"\n\
	rpmver -v 2.0 2.1-1cl\n\
\n\
Written by Gustavo Niemeyer <niemeyer@conectiva.com>\n\
";

int hash(const char *s)
{
	int i = 0;
	while (*s) i += *(s++)+i*3;
	return i;
}

void parsecmdline(char **argv, int argc,
		 const char **ver1, const char **op, const char **ver2)
{
	char *s;
	switch (argc) {
		case 1:
			s = argv[0];
			while (isspace(*s)) s++;
			*ver1 = s;
			while (*s && !isspace(*s)) s++;
			if (!*s) goto error;
			*s++ = 0;
			while (isspace(*s)) s++;
			*op = s;
			while (*s && !isspace(*s)) s++;
			if (!*s) goto error;
			*s++ = 0;
			while (isspace(*s)) s++;
			*ver2 = s;
			while (*s && !isspace(*s)) s++;
			if (!*s) break;
			*s++ = 0;
			while (isspace(*s)) s++;
			if (*s) goto error;
			break;
		case 2:
			*ver1 = argv[0];
			*ver2 = argv[1];
			*op = "";
			break;
		case 3:
			*ver1 = argv[0];
			*op = argv[1];
			*ver2 = argv[2];
			break;
		default:
			goto error;
	}
	return;
error:
	fprintf(stderr, "error: invalid parameters\n");
	exit(2);
}

int main(int argc, char *argv[])
{
	const char *ver1, *op, *ver2;
	int ver1flags = RPMSENSE_EQUAL;
	int ver2flags = RPMSENSE_EQUAL;
	int invert = 0;
	int rc;
#ifndef OLDRPM
	rpmds ver1ds;
	rpmds ver2ds;
#endif

	int verbose = 0;
	const char *verbstr = "";

	int arg = 1;
	while (argv[arg] && argv[arg][0] == '-') {
		switch (hash(argv[arg])) {
			case 4310945: /* --verbose */
			case     298: /* -v */
				verbose = 1;
				break;
			case 4311930: /* --version */
			case   66416: /* --help */
			case     284: /* -h */
				puts(usage);
				return 0;
			default:
				fprintf(stderr, "error: unknown option %s\n",
					argv[arg]);
				exit(2);
		}
		arg++;
	}
	
	parsecmdline(argv+arg, argc-arg, &ver1, &op, &ver2);

	switch (hash(op)) {
		case 301: /* <= */
			ver1flags = RPMSENSE_LESS|RPMSENSE_EQUAL;
			verbstr = "lesser than or equal to";
			break;
		case 309: /* >= */
			ver2flags = RPMSENSE_GREATER|RPMSENSE_EQUAL;
			verbstr = "greater than or equal to";
			break;
		case  60: /* < */
			ver2flags = RPMSENSE_LESS;
			verbstr = "lesser than";
			break;
		case  62: /* > */
			ver2flags = RPMSENSE_GREATER;
			verbstr = "greater than";
			break;
		case 305: /* == */
		case  61: /* = */
			ver2flags = RPMSENSE_EQUAL;
			verbstr = "equal to";
			break;
		case 193: /* != */
		case 302: /* <> */
			ver2flags = RPMSENSE_EQUAL;
			verbstr = "equal to";
			invert = 1;
			break;
		case   0:
			break;
		default:
			fprintf(stderr, "error: unknown operation '%s'\n",
				op);
			exit(2);
	}

#ifndef OLDRPM
	ver1ds = rpmdsSingle(RPMTAG_PROVIDENAME, "", ver1, ver1flags);
	ver2ds = rpmdsSingle(RPMTAG_REQUIRENAME, "", ver2, ver2flags);
	rpmdsSetNoPromote(ver1ds, _rpmds_nopromote);
	rpmdsSetNoPromote(ver2ds, _rpmds_nopromote);
	rc = rpmdsCompare(ver1ds, ver2ds);
	rpmdsFree(ver1ds);
	rpmdsFree(ver2ds);
#else
	rc = rpmRangesOverlap("", ver1, ver1flags, "", ver2, ver2flags);
#endif

	if (!*op) {
		if (rc) {
			if (verbose)
				printf("RPM version %s is equal to "
				       "version %s.\n", ver1, ver2);
			return 0;
		}
#ifndef OLDRPM
		ver1ds = rpmdsSingle(RPMTAG_PROVIDENAME, "", ver1, ver1flags);
		ver2ds = rpmdsSingle(RPMTAG_REQUIRENAME, "", ver2,
				     RPMSENSE_GREATER);
		rpmdsSetNoPromote(ver1ds, _rpmds_nopromote);
		rpmdsSetNoPromote(ver2ds, _rpmds_nopromote);
		rc = rpmdsCompare(ver1ds, ver2ds);
		rpmdsFree(ver1ds);
		rpmdsFree(ver2ds);
#else
		rc = rpmRangesOverlap("", ver1, ver1flags,
				      "", ver2, RPMSENSE_GREATER);
#endif
		if (rc) {
			if (verbose)
				printf("RPM version %s is greater than "
				       "version %s.\n", ver1, ver2);
			return 1;
		}
		if (verbose)
			printf("RPM version %s is lesser than "
			       "version %s.\n", ver1, ver2);
		return -1;
	}
	if (verbose)
		printf("RPM version %s is%s %s version %s.\n",
		       ver1, rc?"":" not", verbstr, ver2);

	return (!invert && !rc) || (invert && rc);
}
